local Mod = ...

function split(s, sep)
    local fields = {}
    
    local sep = sep or " "
    local pattern = string.format("([^%s]+)", sep)
    string.gsub(s, pattern, function(c) fields[#fields + 1] = c end)
    
    return fields
end

TMDefault = {}
BMDefault = {}
i = 0
while (i < 1024/settings["CellSizeX"]) do
	i = i+1
	TMDefault[i] = ""
	BMDefault[i] = ""
	c = 0
	while (c < 1024/settings["CellSizeY"]) do
		c = c +1
		TMDefault[i] = TMDefault[i].."0,"
		BMDefault[i] = BMDefault[i].."false,"
	end
	TMDefault[i] = TMDefault[i]:sub(1, -2)
	BMDefault[i] = BMDefault[i]:sub(1, -2)
end

local COMP_FORAGING_MANAGER = {
	TypeName = "COMP_FORAGING_MANAGER",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "treeMap", Type = "string["..1024/settings["CellSizeX"].."]", Default = TMDefault, Flags = { "SAVE_GAME" }}, 
		{ Name = "bushMap", Type = "string["..1024/settings["CellSizeX"].."]", Default = BMDefault, Flags = { "SAVE_GAME" }}, 
		{ Name = "respawnList", Type = "list<vec3f>", Default = {},  Flags = { "SAVE_GAME" }}, 
--		{ Name = "Counter", Type = "integer", Default = 0, Flags = { "SAVE_GAME" }}, 
		{ Name = "isDestroyBerries", Type = "boolean", Default = settings["DestroyBerries"]}
	}
	
}

function COMP_FORAGING_MANAGER:decodeMap(map)
	T = {}
	for i,str in ipairs(map) do
		T[i] = {}
		for i2, str2 in ipairs(split(str,",")) do
			T[i][i2] = tostring(str2)
		end
	end
	return T
end

function COMP_FORAGING_MANAGER:encodeMap(map)
	T = {}
	for i,a in ipairs(map) do
		T[i] = ""
		for i2,int in ipairs(a) do
			T[i] = T[i]..tostring(int)..","
		end
		T[i] = T[i]:sub(1, -2)
	end
	return T
end

function COMP_FORAGING_MANAGER:spawn(comp,cell)
	offsetX = settings:OffsetX()
	offsetY = settings:OffsetY()
	self:getLevel():createObject(settings["Prefab"],function(Obj)
				Obj:getOrCreateComponent("COMP_GROUNDED")
				Obj:setGlobalPosition({(cell[1]*settings["CellSizeX"])-offsetX,0,(cell[2]*settings["CellSizeY"])-offsetY})
				Obj:setScale(settings["PrefabScale"])
	end)
end
 
function COMP_FORAGING_MANAGER:respawn(comp)
	TreeMap = self:decodeMap(self.treeMap)
	BushMap = self:decodeMap(self.bushMap)
	coords = comp:getOwner():getGlobalPosition()
	cell = {math.ceil(coords[1]/settings["CellSizeX"]),math.ceil(coords[3]/settings["CellSizeY"])}
	if (tonumber(TreeMap[cell[1]][cell[2]]) >= settings["MinimumDensity"]) then
		if (BushMap[cell[1]][cell[2]] == "true") then 
			table.insert(self.respawnList,{settings["RespawnTimerTicks"],cell[1],cell[2]}) 
		end
	else
		BushMap[cell[1]][cell[2]] = "false"
		map = self:encodeMap(BushMap)
		for i,v in ipairs(BushMap) do
			self.bushMap[i] = tostring(map[i])
		end
	end
end

function COMP_FORAGING_MANAGER:TreeUpdate(comp,isExist)
	TreeMap = self:decodeMap(self.treeMap)
	BushMap = self:decodeMap(self.bushMap)
	coords = comp:getOwner():getGlobalPosition()
	cell = {math.ceil(coords[1]/settings["CellSizeX"]),math.ceil(coords[3]/settings["CellSizeY"])}

	if (isExist == true) then
		TreeMap[cell[1]][cell[2]] = tonumber(TreeMap[cell[1]][cell[2]]) +1
		
		map = self:encodeMap(TreeMap)
		for i,v in ipairs(TreeMap) do
			self.treeMap[i] = tostring(map[i])
		end
		
		if (tonumber(TreeMap[cell[1]][cell[2]]) >= settings["MinimumDensity"]) then
			if (BushMap[cell[1]][cell[2]] == "false") then
				self:spawn(comp,cell)
				BushMap[cell[1]][cell[2]] = "true"
				
				map = self:encodeMap(BushMap)
				for i,v in ipairs(BushMap) do
					self.bushMap[i] = tostring(map[i])
				end
			end
		end
	else
		TreeMap[cell[1]][cell[2]] = tonumber(TreeMap[cell[1]][cell[2]]) -1
		
		map = self:encodeMap(TreeMap)
		for i,v in ipairs(TreeMap) do
			self.treeMap[i] = tostring(map[i])
		end

		if (tonumber(TreeMap[cell[1]][cell[2]]) < settings["MinimumDensity"]) then
			if (BushMap[cell[1]][cell[2]] == "true") then
				BushMap[cell[1]][cell[2]] = "false"

				map = self:encodeMap(BushMap)
				for i,v in ipairs(BushMap) do
					self.bushMap[i] = tostring(map[i])
				end
				self:getLevel():getComponentManager("COMP_RESOURCE_CONTAINER"
					):getAllComponent(
					):forEach(function(comp)
						if (comp:getOwner().Name == "BerryBushPart02") then
							cell2 = {math.ceil(comp:getOwner():getGlobalPosition()[1]/settings["CellSizeX"]),math.ceil(comp:getOwner():getGlobalPosition()[3]/settings["CellSizeX"])}
							if (cell[1] == cell2[1]) then
							if(cell[2]==cell2[2])then
								comp:getOwner():destroyAllChild()
								comp:getOwner():destroy()
							end
							end
						end
					end
				)
			end
		end
		
		
	end
end

function COMP_FORAGING_MANAGER:init()
	toDestroy = {}
	
	self:getLevel():getComponentManager("COMP_RESOURCE_CONTAINER"
		):getAllComponent(
		):forEach(function(comp)

			for _,v in ipairs(settings["Destroy"]) do
				if (comp:getOwner().Name == v) then
					if(comp:getOwner().Name == "RedBerries") then
						table.insert(toDestroy,comp:getOwner():getParent():getId())
					else
						table.insert(toDestroy,comp:getOwner():getId())
					end
				end
			end
		end
	)
	
	for _,v in ipairs (toDestroy) do
		obj = self:getLevel():find(v)
		if (obj ~= nil) then
			obj:destroyAllChild()
			obj:destroy()
		end
	end

	self:getLevel():getComponentManager("COMP_TREE"
		):getAllComponent(
		):forEach(function(comp)
			comp:getOwner():getOrCreateComponent("COMP_FORAGING_TREE")
		end
	)	
end

function COMP_FORAGING_MANAGER:update()
t={}
	for i,v in pairs(self.respawnList) do
		if (v[1] <= 0)then
			self:spawn(nil,{v[2],v[3]})
			self.respawnList[i] = nil
		else
			v[1] = v[1] -1
		end
	end
end

--[[
function COMP_FORAGING_MANAGER:onFinalize(isClear)   33005
	TreeMap = self:decodeMap(self.treeMap)
	BushMap = self:decodeMap(self.bushMap)
	i = 0
	while i <32 do
		i = i+1	
		Mod:log(tostring(BushMap[i]))
		Mod:log(tostring(TreeMap[i]))
	end
end
--]]
Mod:registerClass(COMP_FORAGING_MANAGER)

--[[

	if ( self.Counter < 200) then

			self:getLevel():createObject("PREFAB_BERRYBUSH01",function(RedBerries)
				RedBerries:getOrCreateComponent("COMP_GROUNDED")
				RedBerries:setGlobalPosition(SpawnSettings["pos"])
				RedBerries:setScale(5)
				Mod:log(tostring(RedBerries:getGlobalPosition()))

			end)

		self.Counter = self.Counter + 1
	end

--]]